/**************************************************************************
**       Title: int image
**    $RCSfile: SimpleRGBImage.hh,v $
**   $Revision: 1.1 $$Name:  $
**       $Date: 2005/05/02 12:07:11 $
**   Copyright: GPL $Author: fehr $
** Description:
**
**    
**
**-------------------------------------------------------------------------
**
**  $Log: IntImage.hh,v $
**
**
**************************************************************************/

#ifndef INTIMAGE_HH
#define INTIMAGE_HH

#include <iostream>
#include "MyError.hh"


/*======================================================================*/
/*!
 *  \class IntImage  IntImage.hh
 *  \brief The IntImage class is a very basic image class for
 *  storing and manipulating Int images.
 *
 */
/*======================================================================*/
class IntImage
{

    static const unsigned int MAX_DIMENSION = 16384;
    
public:
  /*======================================================================*/
  /*! 
   *   Default Constructor. Creates an empty image
   */
  /*======================================================================*/
  IntImage();

  /*======================================================================*/
  /*! 
   *   Copy Constructor.
   */
  /*======================================================================*/
  IntImage(const IntImage& image);
  
  /*======================================================================*/
  /*! 
   *   Size Constructor. Creates an image of arbitrary dimensions
   *
   *   \param width   width of image
   *   \param height  height of image
   */
  /*======================================================================*/
  IntImage(unsigned int width, unsigned int height);


  /*======================================================================*/
  /*! 
   *   Destructor
   */
  /*======================================================================*/
  ~IntImage();
  
  /*======================================================================*/
  /*! 
   *   resize image to width*height. If it has already this size, nothing
   *   is done 
   *
   *   \param width   width of image
   *   \param height  height of image
   */
  /*======================================================================*/
  void resize(unsigned int width, unsigned int height);
  

  /*======================================================================*/
  /*! 
   *   unchecked access to a pixel (two-dimensional position)
   *
   *   \param x    horizontal pixel coordinate
   *   \param y    vertical pixel coordinate
   *
   *   \return  reference to the pixel at position row,col
   */
  /*======================================================================*/
  int&       operator()(unsigned int x, unsigned int y);
  const int& operator()(unsigned int x, unsigned int y) const;
  


  /*======================================================================*/
  /*! 
   *   width of the image
   *
   *   \return image width
   */
  /*======================================================================*/
  unsigned int width() const;

  
  /*======================================================================*/
  /*! 
   *   height of the image
   *
   *   \return image height
   */
  /*======================================================================*/
  unsigned int height() const;

  /*======================================================================*/
  /*! 
   *   size of the image (number of pixels)
   *
   *   \return number of pixels
   */
  /*======================================================================*/
  size_t size() const;

  /*======================================================================*/
  /*! 
   *   begin of image
   *
   *   \return pointer to first RGBColor of image
   */
  /*======================================================================*/
  int*       begin();
  const int* begin() const;

  /*======================================================================*/
  /*! 
   *   end of image
   *
   *   \return pointer to the first RGBColor BEHIND the last RGBColor 
   *           of the image
   */
  /*======================================================================*/
  int*       end();
  const int* end() const;

  /*======================================================================*/
  /*! 
   *   operator[] 
   *
   *   \param n    pixel number
   *   \return reference to RGBColor number n of image
   */
  /*======================================================================*/
  int&       operator[](size_t n); 
  const int& operator[](size_t n) const;


  /*======================================================================*/
  /*! 
   *   operator=
   *
   *   \param image    image to copy
   *   \return reference to modified image
   */
  /*======================================================================*/
  IntImage& operator=(const IntImage& image);
  
private:
  unsigned int _width, _height, _size;
  int* _pixels;
  int* _pixelsEnd;
  int** _rowStart;
};

#include "IntImage.icc"

#endif
