/**************************************************************************
**       Title: RGBSegment
**    $RCSfile: RGBSegment.cc,v $
**   $Revision: 1.4 $$Name:  $
**       $Date: 2005/05/02 11:25:37 $
**   Copyright: GPL $Author: fehr $
** Description:
**
**                      
**                  
**-------------------------------------------------------------------------
**              
**  $Log: RGBSegment.cc,v $
**              
**                  
**************************************************************************/


#include <iostream>
#include <fstream>
 
#include "MyError.hh"
#include "IntImage.hh"
#include "SimpleRGBImage.hh"
#include "RGBColor.hh"
#include "RGBClassifier.hh"
#include "ConnectedComponentLabeling.hh"



int main(int argc, char** argv)
{
    //check number of arguments
    if (argc != 3)
    {
	std::cerr<<"wrong number of arguments! usage: RGBSegment InFile OutFile \n";
	return 1;
    }
    

    //read image
    std::ifstream infile(argv[1]);
    SimpleRGBImage RGBImage;
    try
    {
	RGBImage.read(infile);
    }
    catch (const MyError& e)
    {
	std::cerr<<e.msg()<<"\n";
    }
    infile.close();
    
    //Binarize image
    RGBClassifier MyClassifier(RGBColor::Red, 10,10,10); //Objects are red, tolerance set
    IntImage* MyIntImage = MyClassifier.classify(RGBImage); //bianrize

    //ConnectedCOmponentLabeling
    ConnectedComponentLabeling MyLabel;
    MyLabel.label(*MyIntImage);

    std::cout<<MyLabel.getNumberOfObjects()<<" objects labeled\n";
   
    SimpleRGBImage OutImage(MyIntImage->width(),MyIntImage->height()); //genarate output image
    for (int n=0;n<MyIntImage->size();n++)
    {
	if (MyIntImage->operator[](n)==0)
	{
	    OutImage[n] = RGBColor::Black;
	}
	else
	{
	switch((int)fmod(MyIntImage->operator[](n),7) )   //color lookup table 
	{
	    case 0:
		OutImage[n] = RGBColor::White;
		break;
	    case 1:
		OutImage[n] = RGBColor::Red;
		break;
	    case 2:
		OutImage[n] = RGBColor::Green;
		break;
	    case 3:
		OutImage[n] = RGBColor::Blue;
		break;
	    case 4:
		OutImage[n] = RGBColor::Yellow;
		break;
	    case 5:
		OutImage[n] = RGBColor::Magenta;
		break;
	    case 6:
		OutImage[n] = RGBColor::Cyan;
		break;
	    
	}
	}
    }
    //save to file
    std::ofstream outfile(argv[2]);
    OutImage.write(outfile);
    outfile.close();
}
